# PE Activity Service

A service for logging activity events.

# Getting Started

NB: This is in an early stage of development.
These instructions are really only useful if you want to develop on the PE Activity Service.

To run the PE Activity Service you will need:

* Postgres
* A JVM
* [Leiningen](http://leiningen.org)

First, create a fresh postgres database for the PE Activity Service.  The following steps should be used:

`su - pe-postgres -s /bin/bash -c /opt/puppet/bin/psql`

```
CREATE USER "pe-activity" PASSWORD '<PASSWORD>';
CREATE DATABASE "pe-activity" OWNER "pe-activity" ENCODING 'utf8' LC_CTYPE 'en_US.utf8' LC_COLLATE 'en_US.utf8' template template0;
```

Edit the `dev-resources/conf.d/activity.conf` configuration file and add a `database` section with the database name, user, and password.
For the example above, assuming `pe-activity` was entered for the user's password, this section should look like:

```
database: {
  dbname: pe-activity,
  user: pe-activity,
  password: pe-activity
}
```

Now you can start the service with:

```
lein dev
```

To initialize database and clear all data run:

```
lein initdb
```

## Running tests

In order to run the tests you need to setup a `activity_test` database owned
by the `activity_test` user, with the password `activity_test`.

You can do this using the following postgres commands (after running `psql` as a postgres superuser):

    CREATE USER "activity_test" PASSWORD 'activity_test';
    CREATE DATABASE "activity_test" OWNER "activity_test" ENCODING 'utf8' LC_CTYPE 'en_US.utf8' LC_COLLATE 'en_US.utf8' template template0;

Once you have created this database, run the tests with:

    lein test

# HTTP API Specification

## GET /v1/events

_Fetches events in a structured JSON format. Supports filtering through query parameters. Web session authentication required._

**Filters:**

* service_id [required]
* subject_type [optional; required only when subject_id is provided]
* subject_id [optional; comma separated list of subject_ids]
* object_type [optional; required only when object_id is provided]
* object_id [optional; comma separated list of object_ids]
* offset=[optional; skip n event commits]
* limit=[optional; return no more than n event commits; defaults to 1000]

**Example response body:**

GET /v1/events?service_id=classifier&subject_type=users&subject_id=frodo

```
{"commits":
 [{"object":"Default Node Group",
   "subject":"Frodo Baggins"
   "timestamp":"2014-06-24T04:00:00Z",
   "events":
   [{"message":"Create Node"},
    {"message":"Create Node Class"}]}],
 "total-rows":1}
```

GET /v1/events?service_id=classifier&object_type=node_groups&object_id=2

```
{"commits":
 [{"object":"Default Node Group",
   "subject":"Frodo Baggins",
   "timestamp":"2014-06-24T04:00:00Z",
   "events":
   [{"message":"Create Node"},
    {"message":"Create Node Class"}]}],
 "total-rows":1}
```

## GET /v1/events.csv

_Fetches events in a flat CSV format. Supports filtering through query parameters. Web session authentication required._

**Filters:**

* service_id=[required]
* subject_type [optional; required only when subject_id is provided]
* subject_id [optional; comma separated list of subject_ids]
* object_type [optional; required only when object_id is provided]
* object_id [optional; comma separated list of object_ids]
* offset=[optional; skip n event commits]
* limit=[optional; return no more than n event commits; defaults to 10000]

**Example response body:**

GET /v1/events.csv?service_id=classifier&subject_type=users&subject_id=frodo

```
Submit Time,Subject Type,Subject Id,Subject Name,Object Type,Object Id,Object Name,Type,What,Description,Message
2014-07-17 13:08:09.985221,users,frodo,Frodo Baggins,node_groups,2,Default Node Group,create,node,create_node,Create Node
2014-07-17 13:08:09.985221,users,frodo,Frodo Baggins,node_groups,2,Default Node Group,create,node_class,create_node_class,Create Node Class
```

# API Specification

## report-activity!

_Stores events._

### Parameters
- event-bundle - event data map in the following format

```
{:commit
 {:service {:id <service id>}
  :subject {:type <subject type> :id <subject id> :name <subject name>}
  :object {:type <object type> :id <object id> :name <object name>}
  :events
  [{:type <event type> :what <event what> :description <event description> :message <event message>}
   ...
   {:type <event type> :what <event what> :description <event description> :message <event message>}]}}
```

**event-bundle example:**
```
{:commit
 {:service {:id "classifier"}
  :subject {:type "users" :id "frodo" :name "Frodo Baggins"}
  :object {:type "node_groups" :id "2" :name "Default Node Group"}
  :events
  [{:type "create" :what "node" :description "create_node" :message "Create Node"}
   {:type "create" :what "node_class" :description "create_node_class" :message "Create Node Class"}]}}
```

### Integration Example

```
(defservice test-service
            [[:ActivityReportingService report-activity!]]

            (start [_ context]
                   (log/info "Starting test service...")
                   (report-activity! {:commit
                                       {:service {:id "classifier"}
                                        :subject {:type "users" :id "frodo" :name "Frodo Baggins"}
                                        :object {:type "node_groups" :id "2" :name "Default Node Group"}
                                        :events
                                        [{:type "create" :what "node" :description "create_node" :message "Create Node"}
                                         {:type "create" :what "node_class" :description "create_node_class" :message "Create Node Class"}]}})
                   (log/info "Finishing test service...")
                   context))
```

# Clear old data

To move old data from DB to a CSV file run the following psql script:

`su - pe-postgres -c "/opt/puppet/bin/psql pe-activity -c \"select backup_activity_events(<threshold>, <filename>);\"" -s /bin/bash`

_WARNING: an existing file will be overwritten._

**Example:**

To export activity events submitted before 2014-06-01 run:

`su - pe-postgres -c "/opt/puppet/bin/psql pe-activity -c \"select backup_activity_events('2014-06-01', '/Users/frodo/pe-activity-service-backup/activities-before-20140601.csv');\"" -s /bin/bash`

**Example output:**

```
Submit Time,Service Id,Subject Type,Subject Id,Subject Name,Object Type,Object Id,Object Name,Type,What,Description,Message
2014-08-11 18:21:46.855903+02,classifier,users,frodo,Frodo Baggins,node_groups,2,Default Node Group,create,node,create_node,Create Node
2014-08-11 18:21:46.855903+02,classifier,users,frodo,Frodo Baggins,node_groups,2,Default Node Group,ßsßcreate,node_class,create_node_class,Create Node Class
```

# RBAC Integration

* to be allowed to inspect action events for an object, user needs to have `view` permission for the corresponding object type and object id
* to be allowed to inspect action events for a subject, user needs to have `view` permission for `activity_log`

